"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_languageserver_1 = require("vscode-languageserver");
const vscode_uri_1 = require("vscode-uri");
const elmMakeDiagnostics_1 = require("./elmMakeDiagnostics");
const elmTestDiagnostics_1 = require("./elmTestDiagnostics");
class DiagnosticsProvider {
    constructor(connection, elmWorkspaceFolder, events, settings, elmAnalyse) {
        this.elmWorkspaceFolder = elmWorkspaceFolder;
        this.getDiagnosticsOnSaveOrOpen = this.getDiagnosticsOnSaveOrOpen.bind(this);
        this.getDiagnosticsOnChange = this.getDiagnosticsOnChange.bind(this);
        this.newElmAnalyseDiagnostics = this.newElmAnalyseDiagnostics.bind(this);
        this.elmMakeIssueToDiagnostic = this.elmMakeIssueToDiagnostic.bind(this);
        this.events = events;
        this.elmAnalyseDiagnostics = elmAnalyse;
        this.connection = connection;
        this.elmMakeDiagnostics = new elmMakeDiagnostics_1.ElmMakeDiagnostics(connection, elmWorkspaceFolder, settings);
        this.elmTestDiagnostics = new elmTestDiagnostics_1.ElmTestDiagnostics(connection, elmWorkspaceFolder, settings);
        this.currentDiagnostics = {
            elmAnalyse: new Map(),
            elmMake: new Map(),
            elmTest: new Map(),
        };
        this.events.on("open", this.getDiagnosticsOnSaveOrOpen);
        this.events.on("change", this.getDiagnosticsOnChange);
        this.events.on("save", this.getDiagnosticsOnSaveOrOpen);
        this.elmAnalyseDiagnostics.on("new-diagnostics", this.newElmAnalyseDiagnostics);
    }
    newElmAnalyseDiagnostics(diagnostics) {
        this.currentDiagnostics.elmAnalyse = diagnostics;
        this.sendDiagnostics();
    }
    sendDiagnostics() {
        const allDiagnostics = new Map();
        for (const [uri, diagnostics] of this.currentDiagnostics.elmMake) {
            allDiagnostics.set(uri, diagnostics);
        }
        for (const [uri, diagnostics] of this.currentDiagnostics.elmTest) {
            const currentDiagnostics = allDiagnostics.get(uri) || [];
            if (currentDiagnostics.length === 0) {
                allDiagnostics.set(uri, diagnostics);
            }
        }
        for (const [uri, diagnostics] of this.currentDiagnostics.elmAnalyse) {
            const currentDiagnostics = allDiagnostics.get(uri) || [];
            if (currentDiagnostics.length === 0) {
                allDiagnostics.set(uri, diagnostics);
            }
        }
        for (const [uri, diagnostics] of allDiagnostics) {
            this.connection.sendDiagnostics({ uri, diagnostics });
        }
    }
    getDiagnosticsOnChange(document) {
        return __awaiter(this, void 0, void 0, function* () {
            this.getDiagnostics(document, false);
        });
    }
    getDiagnosticsOnSaveOrOpen(document) {
        return __awaiter(this, void 0, void 0, function* () {
            this.getDiagnostics(document, true);
        });
    }
    getDiagnostics(document, isSaveOrOpen) {
        return __awaiter(this, void 0, void 0, function* () {
            const uri = vscode_uri_1.URI.parse(document.uri);
            if (uri.toString().startsWith(this.elmWorkspaceFolder.toString())) {
                const text = document.getText();
                if (isSaveOrOpen) {
                    this.currentDiagnostics.elmMake = yield this.elmMakeDiagnostics.createDiagnostics(uri);
                }
                const elmMakeDiagnosticsForCurrentFile = this.currentDiagnostics.elmMake.get(uri.toString());
                if (elmMakeDiagnosticsForCurrentFile &&
                    elmMakeDiagnosticsForCurrentFile.length === 0) {
                    if (isSaveOrOpen) {
                        this.currentDiagnostics.elmTest = yield this.elmTestDiagnostics.createDiagnostics(uri);
                    }
                    this.elmAnalyseDiagnostics.updateFile(uri, text);
                }
                this.sendDiagnostics();
            }
        });
    }
    elmMakeIssueToDiagnostic(issue) {
        const lineRange = vscode_languageserver_1.Range.create(issue.region.start.line === 0
            ? issue.region.start.line
            : issue.region.start.line - 1, issue.region.start.column === 0
            ? issue.region.start.column
            : issue.region.start.column - 1, issue.region.end.line === 0
            ? issue.region.end.line
            : issue.region.end.line - 1, issue.region.end.column === 0
            ? issue.region.end.column
            : issue.region.end.column - 1);
        return vscode_languageserver_1.Diagnostic.create(lineRange, `${issue.overview} - ${issue.details.replace(/\[\d+m/g, "")}`, this.severityStringToDiagnosticSeverity(issue.type), undefined, "Elm");
    }
    severityStringToDiagnosticSeverity(severity) {
        switch (severity) {
            case "error":
                return vscode_languageserver_1.DiagnosticSeverity.Error;
            case "warning":
                return vscode_languageserver_1.DiagnosticSeverity.Warning;
            default:
                return vscode_languageserver_1.DiagnosticSeverity.Error;
        }
    }
}
exports.DiagnosticsProvider = DiagnosticsProvider;
//# sourceMappingURL=diagnosticsProvider.js.map