"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const Diff = __importStar(require("../util/diff"));
const elmUtils_1 = require("../util/elmUtils");
class DocumentFormattingProvider {
    constructor(connection, elmWorkspaceFolder, events, settings) {
        this.connection = connection;
        this.elmWorkspaceFolder = elmWorkspaceFolder;
        this.formatText = (elmFormatPath, text) => __awaiter(this, void 0, void 0, function* () {
            const options = {
                cmdArguments: ["--stdin", "--elm-version 0.19", "--yes"],
                notFoundText: "Install Elm-format via 'npm install -g elm-format",
            };
            const format = elmUtils_1.execCmd(elmFormatPath, options, this.elmWorkspaceFolder, this.connection);
            format.stdin.write(text);
            format.stdin.end();
            const stdout = yield format;
            return Diff.getTextRangeChanges(text, stdout.stdout);
        });
        this.handleFormattingRequest = (params) => __awaiter(this, void 0, void 0, function* () {
            try {
                const settings = yield this.settings.getSettings(this.connection);
                const text = this.events.get(params.textDocument.uri);
                if (!text) {
                    this.connection.console.error("Can't find file for formatting.");
                    return;
                }
                return this.formatText(settings.elmFormatPath, text.getText());
            }
            catch (error) {
                error.message.includes("SYNTAX PROBLEM")
                    ? this.connection.console.error("Running elm-format failed. Check the file for syntax errors.")
                    : this.connection.window.showErrorMessage("Running elm-format failed. Install via " +
                        "'npm install -g elm-format' and make sure it's on your path");
            }
        });
        this.connection = connection;
        this.elmWorkspaceFolder = elmWorkspaceFolder;
        this.events = events;
        this.settings = settings;
        this.connection.onDocumentFormatting(this.handleFormattingRequest);
    }
}
exports.DocumentFormattingProvider = DocumentFormattingProvider;
//# sourceMappingURL=documentFormatingProvider.js.map