"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_languageserver_1 = require("vscode-languageserver");
const elmUtils_1 = require("../util/elmUtils");
const hintHelper_1 = require("../util/hintHelper");
const treeUtils_1 = require("../util/treeUtils");
class HoverProvider {
    constructor(connection, forest, imports) {
        this.connection = connection;
        this.forest = forest;
        this.imports = imports;
        this.handleHoverRequest = (params) => {
            const tree = this.forest.getTree(params.textDocument.uri);
            if (tree) {
                const nodeAtPosition = tree.rootNode.namedDescendantForPosition({
                    column: params.position.character,
                    row: params.position.line,
                });
                const definitionNode = treeUtils_1.TreeUtils.findDefinitionNodeByReferencingNode(nodeAtPosition, params.textDocument.uri, tree, this.imports);
                if (definitionNode) {
                    return this.createMarkdownHoverFromDefinition(definitionNode);
                }
                else {
                    const specialMatch = elmUtils_1.getEmptyTypes().find(a => a.name === nodeAtPosition.text);
                    if (specialMatch) {
                        return {
                            contents: {
                                kind: vscode_languageserver_1.MarkupKind.Markdown,
                                value: specialMatch.markdown,
                            },
                        };
                    }
                }
            }
        };
        this.connection.onHover(this.handleHoverRequest);
    }
    createMarkdownHoverFromDefinition(definitionNode) {
        if (definitionNode) {
            const value = definitionNode.nodeType === "FunctionParameter"
                ? hintHelper_1.HintHelper.createHintFromFunctionParameter(definitionNode.node)
                : hintHelper_1.HintHelper.createHint(definitionNode.node);
            if (value) {
                return {
                    contents: {
                        kind: vscode_languageserver_1.MarkupKind.Markdown,
                        value,
                    },
                };
            }
        }
    }
}
exports.HoverProvider = HoverProvider;
//# sourceMappingURL=hoverProvider.js.map