"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_languageserver_1 = require("vscode-languageserver");
const references_1 = require("../util/references");
const treeUtils_1 = require("../util/treeUtils");
class RenameProvider {
    constructor(connection, forest, imports) {
        this.connection = connection;
        this.forest = forest;
        this.imports = imports;
        this.handleRenameRequest = (params) => __awaiter(this, void 0, void 0, function* () {
            const tree = this.forest.getTree(params.textDocument.uri);
            if (tree) {
                const nodeAtPosition = tree.rootNode.namedDescendantForPosition({
                    column: params.position.character,
                    row: params.position.line,
                });
                const definitionNode = treeUtils_1.TreeUtils.findDefinitionNodeByReferencingNode(nodeAtPosition, params.textDocument.uri, tree, this.imports);
                if (definitionNode) {
                    const refTree = this.forest.getByUri(definitionNode.uri);
                    if (refTree && refTree.writable) {
                        const references = references_1.References.find(definitionNode, this.forest, this.imports);
                        if (references) {
                            const map = {};
                            references.forEach(a => {
                                if (!map[a.uri]) {
                                    map[a.uri] = [];
                                }
                                map[a.uri].push(vscode_languageserver_1.TextEdit.replace(vscode_languageserver_1.Range.create(vscode_languageserver_1.Position.create(a.node.startPosition.row, a.node.startPosition.column), vscode_languageserver_1.Position.create(a.node.endPosition.row, a.node.endPosition.column)), params.newName));
                            });
                            if (map) {
                                return {
                                    changes: map,
                                };
                            }
                        }
                    }
                }
            }
            return undefined;
        });
        this.connection.onRenameRequest(this.handleRenameRequest);
    }
}
exports.RenameProvider = RenameProvider;
//# sourceMappingURL=renameProvider.js.map