"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const cp = __importStar(require("child_process"));
const path = __importStar(require("path"));
const vscode_languageserver_1 = require("vscode-languageserver");
exports.isWindows = process.platform === "win32";
/** Executes a command. Shows an error message if the command isn't found */
function execCmd(cmd, options = {}, elmRootPath, connection) {
    const { onStart, onStdout, onStderr, onExit } = options;
    let childProcess;
    let firstResponse = true;
    let wasKilledByUs = false;
    const executingCmd = new Promise((resolve, reject) => {
        const cmdArguments = options ? options.cmdArguments : [];
        const fullCommand = `${cmd} ${(cmdArguments || []).join(" ")}`;
        childProcess = cp.exec(fullCommand, { cwd: elmRootPath.fsPath }, handleExit);
        if (!childProcess.stdout) {
            return;
        }
        childProcess.stdout.on("data", (data) => {
            if (firstResponse && onStart) {
                onStart();
            }
            firstResponse = false;
            if (onStdout) {
                onStdout(data.toString());
            }
        });
        if (!childProcess.stderr) {
            return;
        }
        childProcess.stderr.on("data", (data) => {
            if (firstResponse && onStart) {
                onStart();
            }
            firstResponse = false;
            if (onStderr) {
                onStderr(data.toString());
            }
        });
        function handleExit(error, stdout, stderr) {
            executingCmd.isRunning = false;
            if (onExit) {
                onExit();
            }
            if (!wasKilledByUs) {
                if (error) {
                    if (options.showMessageOnError) {
                        const cmdName = cmd.split(" ", 1)[0];
                        const cmdWasNotFound = 
                        // Windows method apparently still works on non-English systems
                        (exports.isWindows &&
                            error.message.includes(`'${cmdName}' is not recognized`)) ||
                            (!exports.isWindows && error.code === 127);
                        if (cmdWasNotFound) {
                            const notFoundText = options ? options.notFoundText : "";
                            connection.window.showErrorMessage(`${cmdName} is not available in your path. ${notFoundText}`);
                        }
                        else {
                            connection.window.showErrorMessage(error.message);
                        }
                    }
                    else {
                        reject(error);
                    }
                }
                else {
                    resolve({ stdout, stderr });
                }
            }
        }
    });
    // @ts-ignore
    executingCmd.stdin = childProcess.stdin;
    executingCmd.kill = killProcess;
    executingCmd.isRunning = true;
    return executingCmd;
    function killProcess() {
        wasKilledByUs = true;
        if (exports.isWindows) {
            cp.spawn("taskkill", ["/pid", childProcess.pid.toString(), "/f", "/t"]);
        }
        else {
            childProcess.kill("SIGINT");
        }
    }
}
exports.execCmd = execCmd;
function isTestFile(filename, rootPath) {
    const testFolder = path.join(rootPath, "tests");
    if (filename.startsWith(testFolder)) {
        return true;
    }
    return false;
}
exports.isTestFile = isTestFile;
// Special type that has no core mock https://github.com/elm/compiler/blob/51e20357137ebc9c3f6136cf0a3fe21c24027f39/compiler/src/Canonicalize/Environment/Foreign.hs#L62
function getEmptyTypes() {
    return [
        {
            markdown: `An \`List\` is a list of items. Every item must be of the same type. Valid syntax for lists includes:

    []
    [42, 43]
    ["one", "two", "three"]
    [3.14, 0.1234]
    ['a', 'Z', '0']

    `,
            name: "List",
            symbolKind: vscode_languageserver_1.SymbolKind.Enum,
        },
    ];
}
exports.getEmptyTypes = getEmptyTypes;
//# sourceMappingURL=elmUtils.js.map