import { SyntaxNode, Tree } from "tree-sitter";
import { IImport, IImports } from "../imports";
export declare type NodeType = "Function" | "FunctionParameter" | "TypeAlias" | "Type" | "Operator" | "Module" | "UnionConstructor";
export declare type Exposing = Array<{
    name: string;
    syntaxNode: SyntaxNode;
    type: NodeType;
    exposedUnionConstructors?: Array<{
        name: string;
        syntaxNode: SyntaxNode;
        accessibleWithoutPrefix: boolean;
    }>;
}>;
export declare class TreeUtils {
    static getModuleNameNode(tree: Tree): SyntaxNode | undefined;
    static getModuleNameAndExposing(tree: Tree): {
        moduleName: string;
        exposing: Exposing;
    } | undefined;
    static findFirstNamedChildOfType(type: string, node: SyntaxNode): SyntaxNode | undefined;
    static findAllNamedChildrenOfType(type: string | string[], node: SyntaxNode): SyntaxNode[] | undefined;
    static findExposedFunctionNode(node: SyntaxNode, functionName: string): SyntaxNode | undefined;
    static isExposedFunction(tree: Tree, functionName: string): boolean;
    static findExposedTypeOrTypeAliasNode(node: SyntaxNode, typeName: string): SyntaxNode | undefined;
    static isExposedTypeOrTypeAlias(tree: Tree, typeName: string): boolean;
    static findUnionConstructor(tree: Tree, unionConstructorName: string): SyntaxNode | undefined;
    static findUnionConstructorCalls(tree: Tree, unionConstructorName: string): SyntaxNode[] | undefined;
    static findFunction(tree: Tree, functionName: string): SyntaxNode | undefined;
    static findOperator(tree: Tree, operatorName: string): SyntaxNode | undefined;
    static findTypeDeclaration(tree: Tree, typeName: string): SyntaxNode | undefined;
    static findModuleDeclaration(tree: Tree): SyntaxNode | undefined;
    static findTypeAliasDeclaration(tree: Tree, typeAliasName: string): SyntaxNode | undefined;
    static findAllFunctionDeclarations(tree: Tree): SyntaxNode[] | undefined;
    static findAllTopLeverFunctionDeclarations(tree: Tree): SyntaxNode[] | undefined;
    static findAllTypeOrTypeAliasCalls(tree: Tree): SyntaxNode[] | undefined;
    static findAllFunctionCallsAndParameters(node: SyntaxNode): SyntaxNode[];
    static findAllRecordBaseIdentifiers(node: SyntaxNode): SyntaxNode[];
    static getFunctionNameNodeFromDefinition(node: SyntaxNode): SyntaxNode | undefined;
    static getFunctionAnnotationNameNodeFromDefinition(node: SyntaxNode): SyntaxNode | undefined;
    static getTypeOrTypeAliasNameNodeFromDefinition(node: SyntaxNode): SyntaxNode | undefined;
    static findFunctionCalls(node: SyntaxNode, functionName: string): SyntaxNode[] | undefined;
    static findParameterUsage(node: SyntaxNode, functionName: string): SyntaxNode[] | undefined;
    static findTypeOrTypeAliasCalls(tree: Tree, typeOrTypeAliasName: string): SyntaxNode[] | undefined;
    static findAllTypeDeclarations(tree: Tree): SyntaxNode[] | undefined;
    static findAllTypeAliasDeclarations(tree: Tree): SyntaxNode[] | undefined;
    static findLowercaseQidNode(tree: Tree, nodeAtPosition: SyntaxNode): SyntaxNode | undefined;
    static findUppercaseQidNode(tree: Tree, nodeAtPosition: SyntaxNode): {
        node: SyntaxNode;
        nodeType: NodeType;
    } | undefined;
    static findDefinitionNodeByReferencingNode(nodeAtPosition: SyntaxNode, uri: string, tree: Tree, imports: IImports): {
        node: SyntaxNode;
        uri: string;
        nodeType: NodeType;
    } | undefined;
    static findFunctionParameterDefinition(node: SyntaxNode, functionParameterName: string): SyntaxNode | undefined;
    static findImportFromImportList(uri: string, nodeName: string, type: NodeType, imports: IImports): IImport | undefined;
    static findAllImportNameNodes(tree: Tree): SyntaxNode[] | undefined;
    static findImportClauseByName(tree: Tree, moduleName: string): SyntaxNode | undefined;
    static findImportNameNode(tree: Tree, moduleName: string): SyntaxNode | undefined;
    static getTypeOrTypeAliasOfFunctionParameter(node: SyntaxNode | undefined): SyntaxNode | undefined;
    static getAllFieldsFromTypeAlias(node: SyntaxNode | undefined): Array<{
        field: string;
        type: string;
    }> | undefined;
}
